# Code generated from Pkl module `MyModule`. DO NOT EDIT.
from enum import Enum
from typing import Dict, List, Literal, Optional, Union
from dataclasses import dataclass
import pkll

_NO_DEFAULT = object()

@dataclass
class MyModule:
    foo: str

    _registered_identifier: str = "MyModule"